﻿using FreeRedis;
using MessagePack.Resolvers;
using MessagePack;
using ServiceCenter.Logs;
using System.Collections.Concurrent;
using System.ComponentModel;
using WCS.Core;
using WCS.Entity.Protocol.DataStructure;
using WCS.Entity.Protocol.Station;
using WCS.WorkEngineering.Systems;
using LogInfo = ServiceCenter.Logs.LogInfo;
using System.Diagnostics;

namespace WCS.WorkEngineering.Worlds
{
    /// <summary>
    /// 主世界，所有的系统（交互点）默认在该世界下执行。
    /// 如有系统需独立，请自行增加对应世界
    /// 新增世界应当继承此世界，而不是直接继承World
    /// </summary>
    [Description("主世界")]
    public class MainWorld : WorldWithLog
    {
        public static RedisClient Redis = new RedisClient("");

        /// <summary>
        /// 构造函数
        /// </summary>
        static MainWorld()
        {
            MessagePackSerializer.DefaultOptions = StandardResolver.Options.WithCompression(MessagePackCompression.Lz4BlockArray);
            Redis.Serialize = (obj) =>
            {
                return MessagePackSerializer.Serialize(obj);
            };
            Redis.DeserializeRaw = (data, type) =>
            {

                return MessagePackSerializer.Deserialize(type, data);
            };
        }
         



        /// <summary>
        /// 更新后执行,重写改方法后请自行添加执行内容
        /// 执行内容:清空日志队列
        /// </summary>
        protected override void AfterUpdate(List<WorkTimes> list)
        {
            base.AfterUpdate(list);

            var wt = new WorkTimes();
            wt.Key = "发布监控数据";
            var sw = new Stopwatch();
            sw.Start();
            PublishData();
            sw.Stop();
            wt.Total = sw.ElapsedMilliseconds;
            list.AddSafe(wt);

            var wt1 = new WorkTimes();
            wt1.Key = "主世界保存文本日志";
            var sw1 = new Stopwatch();
            sw1.Start();
            SaveFileLog();
            sw1.Stop();
            wt1.Total = sw.ElapsedMilliseconds;
            list.AddSafe(wt1);

        }
         

        void PublishData()
        {
            try
            {
                var pack = DataCollecctor.Pack; 
                pack.LogContent = WorldWithLog.LogWorlds.SelectMany(v => v.LogList).ToArray();
                var data = pack.GetDeviceData();

                Redis.RPush("Packs", pack);
                if (Redis.LLen("Packs") > 300000)
                {
                    Redis.LTrim("Packs", 5000, -1);
                }
            }
            catch (Exception ex)
            {

            }
        }

            
    }


    public abstract class WorldWithLog : World
    {
        public static List<WorldWithLog> LogWorlds=new List<WorldWithLog>();

        public List<ProtocolData<LogContent>>LogList=new List<ProtocolData<LogContent>>();
        public WorldWithLog()
        {
            LogWorlds.Add(this);
        }
        /// <summary>
        ///  日志队列
        /// </summary>
        protected ConcurrentQueue<KeyLog> Logs = new ConcurrentQueue<KeyLog>();

        /// <summary>
        ///  世界执行周期间隔
        ///  单位：毫秒
        /// </summary>
        protected override int Interval => 600;

        /// <summary>
        ///  更新前执行,重写改方法后请自行添加执行内容
        ///  执行内容:清空日志队列
        /// </summary>
        protected override void BeforeUpdate(List<WorkTimes> list)
        {
            // 清空日志队列,确保日志队列中只会有当前周期日志
            Logs.Clear();
        }

        /// <summary>
        /// 更新后执行,重写改方法后请自行添加执行内容
        /// 执行内容:清空日志队列
        /// </summary>
        protected override void AfterUpdate(List<WorkTimes> list)
        {
            GetLogContents(); 
        }

        void GetLogContents()
        {
            var list = new List<ProtocolData<LogContent>>();
            foreach (var g in Logs.GroupBy(v => v.Channel.Item))
            {
                var log = new ProtocolData<LogContent>();
                log.Code = g.Key;
                log.Data = new LogContent();
                foreach (var g2 in g.GroupBy(v => v.Channel))
                {
                    log.Data.Content += g2.Key + "\n";
                    var arr = g2.Select(v => v.Log.Message).ToArray();
                    log.Data.Content += string.Join("\n", arr);
                    arr = g2.Where(v => v.Log.Level == LogLevelEnum.High).Select(v => v.Log.Message).ToArray();
                    if (arr.Length == 0)
                        log.Data.Error = "";
                    else
                        log.Data.Error += string.Join("。\n", arr) + "。\n";
                }
                list.Add(log);
            }
            LogList = list;
        }


   

        protected virtual void SaveFileLog()
        {
            if (Logs.Count() == 0) return;
            var gs = Logs.GroupBy(v => new { World = v.Channel.World, Stage = v.Channel.Stage, System = v.Channel.System }).ToArray();
            Parallel.ForEach(gs, sys =>
            {
                var time = $"{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss:FF")}";
                var fst = Logs.First();
                var dir = $"Logs\\{fst.Time.ToString("yyyy-MM-dd")}\\{sys.Key.World}-{sys.Key.Stage}-{sys.Key.System}\\";
                if (!Directory.Exists(dir)) Directory.CreateDirectory(dir);
                var ds = sys.GroupBy(v => v.Channel).ToArray();
                Parallel.ForEach(ds, d =>
                {
                    var msg = $"\n-------------{time}-------------\n" + string.Join('\n', d.Select(v => v.Log.Message));
                    File.AppendAllText(Path.Combine(dir, $"{d.Key.Item}.txt"), msg);
                });
                //foreach (var d in ds)
                //{
                //}
            });
            
        }

         


        /// <summary>
        ///  异常处理,重写改方法后请自行添加执行内容
        ///  执行内容:Exception as KnownException并添加至日志队列
        /// </summary>
        /// <param name="channel"></param>
        /// <param name="exception"></param>
        /// <exception cref="NotImplementedException"></exception>
        protected override void OnError(Channel channel, Exception exception)
        {
            if (exception is KnownException)
            {
                var ex = exception as KnownException;
                var log = new LogInfo { Level = ex.Level, Type = ErrorTypeEnum.Kown, LogUpLoad = ex.logUpLoad, Message = ex.Message };
                Logs.Enqueue(new KeyLog { Channel = channel, Log = log, Time = DateTime.Now });
            }
            else
            {
                var log = new LogInfo { Level = LogLevelEnum.High, Type = ErrorTypeEnum.Unkown, LogUpLoad = LogUpLoadEnum.UpLoadWMS, Message = exception.Message };
                Logs.Enqueue(new KeyLog { Channel = channel, Log = log, Time = DateTime.Now });
            }
        }

        /// <summary>
        ///  日志处理,重写改方法后请自行添加执行内容
        ///  执行内容:LogInfo as KeyLog并添加至日志队列
        /// </summary>
        /// <param name="channel"></param>
        /// <param name="logObj"></param>
        /// <exception cref="NotImplementedException"></exception>
        protected override void OnLog(Channel channel, object logObj)
        {
            var log = (LogInfo)logObj;
            Logs.Enqueue(new KeyLog { Channel = channel, Log = log, Time = DateTime.Now });
        }

        /// <summary>
        ///  日志处理,重写改方法后请自行添加执行内容
        /// </summary>
        /// <param name="channel"></param>
        /// <param name="msg"></param>
        /// <exception cref="NotImplementedException"></exception>
        protected override void OnInternalLog(Channel channel, string msg)
        {
            var log = new LogInfo { Level = LogLevelEnum.Low, Message = msg };
            //if (msg != "开始" && msg != "结束")
            {
                Logs.Enqueue(new KeyLog { Channel = channel, Log = log, Time = DateTime.Now });
            }
        }

        /// <summary>
        ///  获取日志,重写改方法后请自行添加执行内容
        /// </summary>
        /// <param name="channel"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        protected override IEnumerable<string> GetChannelMsg(Channel channel)
        {
            return Logs.Where(v => v.Channel.ToString() == channel.ToString()).Select(v => v.Log.Message);
        }
    }

}
